"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

exports.default = function (babel) {
  var t = babel.types,
      template = babel.template;


  return {
    name: "ast-transform", // not required
    visitor: {
      Program: function Program(path, state) {
        var fileName = state.opts.fileName || "no-file";

        // Default imported variable name to "styled", adjust based on import below
        var importedVariableName = 'styled';

        var isStyled = function isStyled(tag) {
          return tag.object && tag.object.name === importedVariableName || tag.callee && tag.callee.name === importedVariableName;
        };

        path.traverse({
          ImportDeclaration: function ImportDeclaration(path) {
            // Is the styled-components import!
            if (path.node.source.value === 'styled-components') {
              // If the default is imported it's at defaultImport[0], otherwise defaultImport is empty
              var defaultImport = path.get('specifiers').find(function (specifier) {
                return specifier.isImportDefaultSpecifier() || specifier.isImportSpecifier() && specifier.node.imported.name === 'default';
              });
              if (defaultImport) {
                // Save the imported name
                importedVariableName = defaultImport.node.local.name;
              }
            }
          }
        });

        path.traverse({
          VariableDeclarator: function VariableDeclarator(path2) {

            path2.traverse({
              TaggedTemplateExpression: function TaggedTemplateExpression(path3) {
                var tag = path3.node.tag;


                if (tag.type === "CallExpression" && tag.callee.name === importedVariableName || tag.type === "MemberExpression" && tag.object.name === importedVariableName) {
                  path3.traverse({
                    TemplateLiteral: function TemplateLiteral(path4) {
                      var expressions = path4.node.expressions;

                      for (var i = 0; i < expressions.length; i++) {
                        var expression = expressions[i];
                        if (expression.type === "Identifier") {
                          expressions[i] = t.conditionalExpression(t.memberExpression(expression, t.identifier("selector")), t.memberExpression(expression, t.identifier("selector")), expression);
                        }
                      }
                    }
                  });

                  var _path3$node$loc = path3.node.loc,
                      start = _path3$node$loc.start,
                      end = _path3$node$loc.end;

                  path3.replaceWith(template("(function(){\n                    let x = React.createFactory(ORIG)\n                    class y extends React.Component {\n                      render(){\n                        let {className} = this.props\n                      \tclassName = className?(className+\" \"+CLASS_NAME):className\n\n                    \t  return x(Object.assign({}, this.props, {[\"className\"]: className}))\n                      }\n                      static get selector(){\n                        return \".\" + CLASS_NAME\n                      }\n                    }\n              \t\t\treturn y\n              \t\t})()")({
                    ORIG: path3,
                    CLASS_NAME: t.stringLiteral("s-" + (0, _hash2.default)(fileName + "-" + start.line + "_" + start.column + "__" + end.line + "_" + end.column))
                  }));

                  path2.skip();
                  path3.skip();
                }
              }
            });
          }
        });
      }
    }
  };
};

var _hash = require("./hash");

var _hash2 = _interopRequireDefault(_hash);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }